@@ -4,8 +4,10 @@ Tests for L{pyzim.pointerlist}.
 import io
 import struct
 import unittest
+from unittest import mock
 
-from pyzim.pointerlist import SimplePointerList, OrderedPointerList
+from pyzim.pointerlist import SimplePointerList, OrderedPointerList, TitlePointerList
+from pyzim.pointerlist import OnDiskSimplePointerList, OnDiskOrderedPointerList, OnDiskTitlePointerList
 from pyzim import constants, exceptions
 
 from .base import TestBase
@@ -19,6 +21,14 @@ class SimplePointerListTests(unittest.TestCase, TestBase):
         self.data = [10, 11, 12, 13, 14, 15]
         self.rawlist = struct.pack(constants.ENDIAN + SimplePointerList.POINTER_FORMAT * 6, *self.data)
 
+    def test_new(self):
+        """
+        Test L{pyzim.pointerlist.SimplePointerList.new}.
+        """
+        pointerlist = SimplePointerList.new()
+        self.assertEqual(len(pointerlist), 0)
+        self.assertEqual(pointerlist._pointers, [])
+
     def test_simple_read(self):
         """
         Test a simple read operation.
@@ -32,55 +42,6 @@ class SimplePointerListTests(unittest.TestCase, TestBase):
             pointerlist.get_by_index(len(pointerlist))  # out of range
         self.assertFalse(pointerlist.dirty)
 
-    def test_modify(self):
-        """
-        Test modification operations.
-        """
-        pointerlist = SimplePointerList.from_bytes(self.rawlist)
-        self.assertEqual(len(pointerlist), 6)
-        self.assertFalse(pointerlist.dirty)
-        for i in range(len(pointerlist)):
-            self.assertEqual(pointerlist.get_by_index(i), i + 10)
-        pointerlist.set(1, 1000)
-        self.assertEqual(pointerlist.get_by_index(1), 1000)
-        self.assertTrue(pointerlist.dirty)
-        pointerlist.dirty = False
-        pointerlist.append(2000)
-        self.assertEqual(pointerlist.get_by_index(len(pointerlist) - 1), 2000)
-        self.assertTrue(pointerlist.dirty)
-        # test set() with index >= len(pointerlist)
-        pointerlist.dirty = False
-        # i == len(pointerlist) should append value
-        pointerlist.set(len(pointerlist), 10000)
-        self.assertEqual(len(pointerlist), 8)
-        self.assertEqual(pointerlist.get_by_index(7), 10000)
-        self.assertTrue(pointerlist.dirty)
-        # i > len with add_placeholders=False should raise an exception
-        pointerlist.dirty = False
-        with self.assertRaises(IndexError):
-            pointerlist.set(12, 11000, add_placeholders=False)
-        self.assertEqual(len(pointerlist), 8)
-        self.assertEqual(pointerlist.get_by_index(7), 10000)
-        self.assertFalse(pointerlist.dirty)
-        # i > len with add_placeholders=True should add placeholders
-        pointerlist.dirty = False
-        pointerlist.set(12, 11000, add_placeholders=True)
-        self.assertEqual(len(pointerlist), 13)
-        self.assertEqual(pointerlist.get_by_index(10), 11000)
-        self.assertEqual(pointerlist.get_by_index(12), 11000)
-        self.assertTrue(pointerlist.dirty)
-        # ensure an error will be raised if not mutable
-        pointerlist.mutable = False
-        pointerlist.dirty = False
-        with self.assertRaises(exceptions.NonMutable):
-            pointerlist.append(3000)
-        self.assertFalse(pointerlist.dirty)
-        self.assertEqual(len(pointerlist), 13)
-        with self.assertRaises(exceptions.NonMutable):
-            pointerlist.set(1, 500)
-        self.assertFalse(pointerlist.dirty)
-        self.assertEqual(pointerlist.get_by_index(1), 1000)
-
     def test_from_file(self):
         """
         Test parsing of a file.
@@ -90,6 +51,27 @@ class SimplePointerListTests(unittest.TestCase, TestBase):
         self.assertEqual(len(pointerlist), 6)
         for i in range(len(pointerlist)):
             self.assertEqual(pointerlist.get_by_index(i), i + 10)
+        # repeat without seek parameter
+        f.seek(4)
+        pointerlist = SimplePointerList.from_file(f, len(self.data))
+        self.assertEqual(len(pointerlist), 6)
+        for i in range(len(pointerlist)):
+            self.assertEqual(pointerlist.get_by_index(i), i + 10)
+
+    def test_from_zim_entry(self):
+        """
+        Test L{pyzim.pointerlist.SimplePointerList.from_zim_entry}.
+        """
+        with self.open_zts_zim("nons", "small.zim") as zim:
+            # we've got a bit of a problem here: there's no simple
+            # pointer list stored inside an entry of any ZIM file,
+            # only title pointer lists. To test this anyway, we are going
+            # to overwrite the pointer format for this test
+            with mock.patch("pyzim.pointerlist.SimplePointerList.POINTER_FORMAT", TitlePointerList.POINTER_FORMAT):
+                pointerlist = SimplePointerList.from_zim_entry(zim, constants.URL_ENTRY_TITLE_INDEX)
+                pointers = list(pointerlist.iter_pointers())
+                orgpointers = list(zim._entry_title_pointer_list.iter_pointers())
+                self.assertEqual(pointers, orgpointers)
 
     def test_to_bytes(self):
         """
@@ -147,6 +129,55 @@ class SimplePointerListTests(unittest.TestCase, TestBase):
         with self.assertRaises(KeyError):
             pointerlist.get_by_pointer(35)
 
+    def test_modify(self):
+        """
+        Test modification operations.
+        """
+        pointerlist = SimplePointerList.from_bytes(self.rawlist)
+        self.assertEqual(len(pointerlist), 6)
+        self.assertFalse(pointerlist.dirty)
+        for i in range(len(pointerlist)):
+            self.assertEqual(pointerlist.get_by_index(i), i + 10)
+        pointerlist.set(1, 1000)
+        self.assertEqual(pointerlist.get_by_index(1), 1000)
+        self.assertTrue(pointerlist.dirty)
+        pointerlist.dirty = False
+        pointerlist.append(2000)
+        self.assertEqual(pointerlist.get_by_index(len(pointerlist) - 1), 2000)
+        self.assertTrue(pointerlist.dirty)
+        # test set() with index >= len(pointerlist)
+        pointerlist.dirty = False
+        # i == len(pointerlist) should append value
+        pointerlist.set(len(pointerlist), 10000)
+        self.assertEqual(len(pointerlist), 8)
+        self.assertEqual(pointerlist.get_by_index(7), 10000)
+        self.assertTrue(pointerlist.dirty)
+        # i > len with add_placeholders=False should raise an exception
+        pointerlist.dirty = False
+        with self.assertRaises(IndexError):
+            pointerlist.set(12, 11000, add_placeholders=False)
+        self.assertEqual(len(pointerlist), 8)
+        self.assertEqual(pointerlist.get_by_index(7), 10000)
+        self.assertFalse(pointerlist.dirty)
+        # i > len with add_placeholders=True should add placeholders
+        pointerlist.dirty = False
+        pointerlist.set(12, 11000, add_placeholders=True)
+        self.assertEqual(len(pointerlist), 13)
+        self.assertEqual(pointerlist.get_by_index(10), 11000)
+        self.assertEqual(pointerlist.get_by_index(12), 11000)
+        self.assertTrue(pointerlist.dirty)
+        # ensure an error will be raised if not mutable
+        pointerlist.mutable = False
+        pointerlist.dirty = False
+        with self.assertRaises(exceptions.NonMutable):
+            pointerlist.append(3000)
+        self.assertFalse(pointerlist.dirty)
+        self.assertEqual(len(pointerlist), 13)
+        with self.assertRaises(exceptions.NonMutable):
+            pointerlist.set(1, 500)
+        self.assertFalse(pointerlist.dirty)
+        self.assertEqual(pointerlist.get_by_index(1), 1000)
+
     def test_remove_by_index(self):
         """
         Test L{pyzim.pointerlist.SimplePointerList.remove_by_index}.
@@ -287,6 +318,17 @@ class OrderedPointerListTests(unittest.TestCase, TestBase):
         self.assertEqual(pointerlist.get("g"), 5)
         self.assertFalse(pointerlist.has("test"))
         pointerlist.check_sorted()
+        # repeat without seek parameter
+        f = io.BytesIO(b"test" + self.rawlist + b"test")
+        f.seek(4)
+        pointerlist = OrderedPointerList.from_file(f, len(self.data), key_func=self.keyfunc)
+        self.assertEqual(len(pointerlist), 6)
+        self.assertEqual(pointerlist.get("a"), 0)
+        self.assertEqual(pointerlist.get("c"), 2)
+        self.assertEqual(pointerlist.get("e"), 3)
+        self.assertEqual(pointerlist.get("g"), 5)
+        self.assertFalse(pointerlist.has("test"))
+        pointerlist.check_sorted()
 
     def test_add_remove(self):
         """
@@ -435,3 +477,114 @@ class OrderedPointerListTests(unittest.TestCase, TestBase):
             self.assertEqual(len(values), rl_end - rl_start)
             for v_a, v_b in zip(values, self.data[rl_start:rl_end]):
                 self.assertEqual(v_a, v_b)
+
+
+class OnDiskSimplePointerListTests(unittest.TestCase, TestBase):
+    """
+    Tests for L{pyzim.pointerlist.OnDiskSimplePointerlist}.
+    """
+    def test_raise_unsupported_init(self):
+        """
+        Test that various initializers not supported by L{pyzim.pointerlist.OnDiskSimplePointerlist} raise the appropiate error.
+        """
+        with self.open_zts_small() as zim:
+            with self.assertRaises(exceptions.OperationNotSupported):
+                OnDiskSimplePointerList.new()
+            with self.assertRaises(exceptions.OperationNotSupported):
+                OnDiskSimplePointerList.from_bytes(b"")
+            with self.assertRaises(exceptions.OperationNotSupported):
+                with zim.acquire_file() as f:
+                    OnDiskSimplePointerList.from_file(f, zim.header.cluster_count, seek=zim.header.cluster_pointer_position)
+
+    def test_from_zim_file(self):
+        """
+        Test L{pyzim.pointerlist.OnDiskSimplePointerList.from_zim_file}
+        """
+        # attempt to manually locate a cluster for the main page entry
+        with self.open_zts_small() as zim:
+            entry = zim.get_mainpage_entry().resolve()
+            cluster = entry.get_cluster()
+            pointerlist = OnDiskSimplePointerList.from_zim_file(zim, n=zim.header.cluster_count, seek=zim.header.cluster_pointer_position)
+            self.assertEqual(pointerlist.get_by_index(entry.cluster_number), cluster.offset)
+        # repeat without seek parameter
+        with self.open_zts_small() as zim:
+            entry = zim.get_mainpage_entry().resolve()
+            cluster = entry.get_cluster()
+            with zim.acquire_file() as f:
+                f.seek(zim.header.cluster_pointer_position)
+            pointerlist = OnDiskSimplePointerList.from_zim_file(zim, n=zim.header.cluster_count)
+            self.assertEqual(pointerlist.get_by_index(entry.cluster_number), cluster.offset)
+
+    def test_from_zim_entry(self):
+        """
+        Test L{pyzim.pointerlist.OnDiskSimplePointerList.from_zim_entry}
+        """
+        with self.open_zts_zim("nons", "small.zim") as zim:
+            # we've got a bit of a problem here: there's no simple
+            # pointer list stored inside an entry of any ZIM file,
+            # only title pointer lists. To test this anyway, we are going
+            # to overwrite the pointer format for this test
+            with mock.patch("pyzim.pointerlist.OnDiskSimplePointerList.POINTER_FORMAT", TitlePointerList.POINTER_FORMAT):
+                pointerlist = OnDiskSimplePointerList.from_zim_entry(zim, constants.URL_ENTRY_TITLE_INDEX)
+                pointers = list(pointerlist.iter_pointers())
+                orgpointers = list(zim._entry_title_pointer_list.iter_pointers())
+                self.assertEqual(pointers, orgpointers)
+
+
+class OnDiskOrderedPointerListTests(unittest.TestCase, TestBase):
+    """
+    Tests for L{pyzim.pointerlist.OnDiskOrderedPointerlist}.
+    """
+    def test_raise_unsupported_init(self):
+        """
+        Test that various initializers not supported by L{pyzim.pointerlist.OnDiskOrderedPointerlist} raise the appropiate error.
+        """
+        with self.open_zts_small() as zim:
+            with self.assertRaises(exceptions.OperationNotSupported):
+                OnDiskOrderedPointerList.new(key_func=lambda x: 0)
+            with self.assertRaises(exceptions.OperationNotSupported):
+                OnDiskOrderedPointerList.from_bytes(b"", key_func=lambda x: 0)
+            with self.assertRaises(exceptions.OperationNotSupported):
+                with zim.acquire_file() as f:
+                    OnDiskOrderedPointerList.from_file(f, zim.header.entry_count, seek=zim.header.cluster_pointer_position, key_func=lambda x: 0)
+
+    def test_from_zim_file(self):
+        """
+        Test L{pyzim.pointerlist.OnDiskOrderedPointerList.from_zim_file}
+        """
+        # attempt to locate main entry
+        with self.open_zts_small() as zim:
+            pointerlist = OnDiskOrderedPointerList.from_zim_file(
+                zim,
+                n=zim.header.entry_count,
+                seek=zim.header.url_pointer_position,
+                key_func=zim._get_full_url_for_entry_at,
+            )
+            self.assertEqual(len(pointerlist), zim.header.entry_count)
+            pointerlist.check_sorted()
+        # repeat without seek parameter
+        with self.open_zts_small() as zim:
+            with zim.acquire_file() as f:
+                f.seek(zim.header.url_pointer_position)
+            pointerlist = OnDiskOrderedPointerList.from_zim_file(
+                zim,
+                n=zim.header.entry_count,
+                key_func=zim._get_full_url_for_entry_at,
+            )
+            self.assertEqual(len(pointerlist), zim.header.entry_count)
+            pointerlist.check_sorted()
+
+    def test_from_zim_entry(self):
+        """
+        Test L{pyzim.pointerlist.OnDiskOrderedPointerList.from_zim_entry}
+        """
+        with self.open_zts_zim("nons", "small.zim") as zim:
+            pointerlist = OnDiskTitlePointerList.from_zim_entry(
+                zim,
+                constants.URL_ENTRY_TITLE_INDEX,
+                key_func=zim._get_namespace_title_for_entry_by_url_index,
+            )
+            pointers = list(pointerlist.iter_pointers())
+            orgpointers = list(zim._entry_title_pointer_list.iter_pointers())
+            self.assertEqual(pointers, orgpointers)
+            pointerlist.check_sorted()
